#include "stdafx.h"
#include <vector>
#include <algorithm> 
#include <fstream>
using namespace std;

#include "diagnostic.h"
#include "findbreaks.h"
#include "typed.h"

#include "reversal\blend.h"
#include "reversal\woven.h"
#include "reversal\weightedaverage.h"
map<string, int> calls;
//TimeFile timed;

/*//INITIAL calls on 440 field clip:
2013			FindNumberBreaks::interval
56364		FindBreaks::interval			x28... required (number of thresholds)
1409100		FindBreaks::unitinterval		x25... required (size of interval)
3247536		LocalAnalyse::blendmatch	x2... effectively x6 as similar number calls to two other methods
36988650	ComputeDistance::distance	x11... effectively x2 which is ok
So the number of calls to FindNumberBreaks::interval must be reduced or something must be cached

CACHING result of	 FindNumberBreaks::interval
2013	FindNumberBreaks::interval
476	FindBreaks::interval
11900	FindBreaks::unitinterval
27454	LocalAnalyse::blendmatch
312445	ComputeDistance::distance
*/


class InferTwelveBlend : public Blend
//gives fields in terms of original FRAME numbers
{
	//Distance *distance;
	LocalAnalyse *analyse;
	int noblocks;
	FindNumberBreaks findnumberbreaks;
public:
	InferTwelveBlend(LocalAnalyse *_analyse) 
		: Blend (25.0/12.0), analyse(_analyse), noblocks(_analyse->frames()/25),
		findnumberbreaks(analyse, 2.9, 12) {}
	//~InferTwelveBlend() ;
	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *env) /*const*/;
};

DisplacedVector<double> InferTwelveBlend::weight(int n, IScriptEnvironment *env) /*const*/
{
	int block = n/25;
	if(block >= noblocks)
	{
		int index = noblocks*12 + (n - noblocks*25)/2;
		return DisplacedVector<double>(index, index+1, 1.0);
	}
//timed.reset();
	list<double> breaks = findnumberbreaks.interval(block*25, (block+1)*25, env);
//timed<< " Calculating blendweight";

	int firstframe = block*12;
	list<double>::iterator j = breaks.begin(); 
	while(j != breaks.end() && *j <= n)
	{
		++firstframe;
		++j;
	}

	if(j != breaks.end() && int(*j) == n)//the next break happens in the middle of frame n
	{
		DisplacedVector<double> result(firstframe, firstframe+2);
		double fraction = *j - int(*j);
		//a small value of fraction means that its very like the next frame...
		result[firstframe] = fraction;
		result[firstframe+1] = 1-fraction;
		return result;
	}
	return DisplacedVector<double>(firstframe, firstframe+1, 1.0);
}


class DistanceBuffer : public GenericVideoFilter 
{   
	const int lead;
	const double ratio;//MAGIC NUMBERS

	Distance *distance;
	int nextcompute;//frames 0...nextcompute-1 have been computed.

	void computeto(int n, IScriptEnvironment *env);
public:
	DistanceBuffer(Distance *_distance, PClip clip, IScriptEnvironment *env);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
};

DistanceBuffer::DistanceBuffer(Distance *_distance, PClip clip, IScriptEnvironment *env)
: GenericVideoFilter(clip), distance(_distance), nextcompute(0),
lead(25), ratio(25.0/24.0)
{
	computeto(lead-1, env);
}

void DistanceBuffer::computeto(int n, IScriptEnvironment *env) 
{
	for(int i = nextcompute; i <= n; ++i)
	{
		if(i+1 < distance->frames())
			distance->distance(i, i+1, env);
		if(i+2 < distance->frames())
			distance->distance(i, i+2, env);
	}
	nextcompute = max(nextcompute, n+1);
}

PVideoFrame __stdcall DistanceBuffer::GetFrame(int n, IScriptEnvironment *env) 
{
	computeto(int(n*ratio) + lead-1, env);
	return child->GetFrame(n, env);
}

class TestDistance : public GenericVideoFilter 
{   
	Distance *distance;
public:
	TestDistance(Distance *_distance, PClip clip, IScriptEnvironment *env)
		: GenericVideoFilter(clip), distance(_distance) {}
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
};
PVideoFrame __stdcall TestDistance::GetFrame(int i, IScriptEnvironment *env) 
{
	if(i+1 < distance->frames())
		distance->distance(i, i+1, env);
	if(i+2 < distance->frames())
		distance->distance(i, i+2, env);
	return child->GetFrame(i, env);
}

//
//class AverageCopies : public GenericVideoFilter //essentially an interleave
//{   
//	Blend *blend;
//public:
//	AverageCopies(PClip, Blend *, IScriptEnvironment *);
//	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *);
//};
//
//AverageCopies::AverageCopies(PClip clip, Blend *_blend, IScriptEnvironment *env)
//	: GenericVideoFilter(clip), blend(_blend)
//{
//}
//PVideoFrame __stdcall AverageCopies::GetFrame(int n, IScriptEnvironment *env)
//{
//	if(blend->weight(n, env).size() != 1) //only deal with unblended frames
//		return child->GetFrame(n, env);
//	int copiedframe = blend->weight(n, env).beginindex();
//
//	int firstcopy = n;
//	while(  firstcopy-1 >= 0 && 
//		blend->weight(firstcopy-1, env).size() == 1 && 
//		blend->weight(firstcopy-1, env).beginindex() == copiedframe )
//		--firstcopy;
//
//	int lastcopy = n;
//	while(  lastcopy+1 < vi.num_frames &&
//		blend->weight(lastcopy+1, env).size() == 1 && 
//		blend->weight(lastcopy+1, env).beginindex() == copiedframe )
//		++lastcopy;
//
//	//if(lastcopy ==  firstcopy)
//		return child->GetFrame(n, env);
//
//	DisplacedVector<double> v(firstcopy, lastcopy+1, 1.0/(lastcopy+1 - firstcopy));
//	return WeightedAverage(child, v, env, vi);
//}

Distance *distancesource(AVSValue value, IScriptEnvironment *env)
{
	if(value.IsClip())
		return new ComputeDistance(value.AsClip()); 
	if(value.IsString())
	{
		ifstream input(value.AsString());
		if(!input)
			env->ThrowError("ReverseBlend: invalid input file.");
		return new ReadDistance(input); 
	}
	env->ThrowError("ReverseBlend: Invalid analysis source (clip or filename required).");
	throw;
}

AVSValue __cdecl test(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	Distance *d = distancesource(args[1], env);
	PClip source = args[0].AsClip();
	//return source;
	return new TestDistance(d,source, env);

}

AVSValue __cdecl writecalls(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	ofstream output("calls.txt");
	for(map<string, int>::const_iterator i = calls.begin(); i != calls.end(); ++i)
		output<<i-> second << "\t" <<i-> first<< "\n";
	return 0;
}

AVSValue __cdecl writeblenddata(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
//timed<< "Before WriteBlendData";
	ofstream output(args[1].AsString("written blend data.txt"));
	write(output, ComputeDistance(args[0].AsClip()), env);
//timed<< "after WriteBlendData";
	return 0;
}

//AVSValue __cdecl copyblenddata(AVSValue args, void* user_data, IScriptEnvironment *env) 
//{
//	ifstream input("written blend data.txt");
//	ofstream output("copied written blend data.txt");
//	write(output, ReadDistance(input), env);
//	return 0;
//}

AVSValue __cdecl call_blendlist(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	PClip clip = args[0].AsClip();
	ofstream output(args[1].AsString("blend data for reader.txt"));
	blendlist(clip, output, env);
	return clip;
}

AVSValue __cdecl call_blendtable(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	blendtable(args[0].AsClip(), env);
	return 0;
}

AVSValue __cdecl call_writebreaks(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	writebreaks(args[0].AsClip(), env);
	return 0;
}

AVSValue __cdecl Create_Reverse12Breaks(AVSValue args, void *user_data, IScriptEnvironment *env) 
{
	Distance *d = distancesource(args[1], env);
	LocalAnalyse *analyse = new LocalAnalyse(d);
	bool buffer = args[2].AsBool(false);
	PClip result = new ReverseWovenBlend(args[0].AsClip(), new InferTwelveBlend(analyse), env);
	if(buffer) 
		result = new DistanceBuffer(d, result, env);
	return result;
}

AVSValue __cdecl Create_ShowBreaks(AVSValue args, void *user_data, IScriptEnvironment *env) 
{
	return new ShowBreaks(  new LocalAnalyse(distancesource(args[0], env)), args[1].AsClip(), env  );  
}
AVSValue __cdecl Create_Show12Breaks(AVSValue args, void *user_data, IScriptEnvironment *env) 
{
	return new Show12Breaks(  new LocalAnalyse(distancesource(args[0], env)), args[1].AsClip(), env  );  
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment *env) 
{
	env->AddFunction("BlendList", "c[filename]s", call_blendlist, 0);
	env->AddFunction("BlendTable", "c", call_blendtable, 0);
	//env->AddFunction("CopyBlendData", "c", copyblenddata, 0);
	env->AddFunction("WriteBlendData", "c[filename]s", writeblenddata, 0);
	env->AddFunction("WriteBreaks", "c", call_writebreaks, 0);

	env->AddFunction("ExperimentalReversalAlpha", "c.[buffer]b", Create_Reverse12Breaks, 0);
	env->AddFunction("Show12Breaks", ".c", Create_Show12Breaks, 0);
	env->AddFunction("ShowBreaks", ".c", Create_ShowBreaks, 0);

	env->AddFunction("Test", "cc", test, 0);

	env->AddFunction("BlendAngle", "c", Create_BlendAngle, 0);
	env->AddFunction("BlendWeight", "c", Create_BlendWeight, 0);

	env->AddFunction("Minimum", "c+", Create_Minimum, 0);
	env->AddFunction("Min", "c+", Create_Minimum, 0);
	env->AddFunction("Maximum", "c+", Create_Maximum, 0);
	env->AddFunction("Max", "c+", Create_Maximum, 0);
	env->AddFunction("WriteNumber", "c[filename]s", Create_WriteNumber, 0);
	env->AddFunction("IsAbove", "cf", Create_IsAbove, 0);	
	env->AddFunction("IsBelow", "cf", Create_IsBelow, 0);
	env->AddFunction("IsEqualTo", "cf", Create_IsEqualTo, 0);
	env->AddFunction("And", "cc", Create_And, 0);
	env->AddFunction("Or", "cc", Create_Or, 0);

	return "BlendTable plugin";
}
