#include "stdafx.h"

#include "diagnostic.h"
#include <fstream>
using namespace std;

ShowBreaks::ShowBreaks(LocalAnalyse *_analyse, PClip _show, IScriptEnvironment *env)
: GenericVideoFilter(_show), analyse(_analyse)
{
	//we need to be working with complete blocks of 25...
	//vi.num_frames = (_toanalyse->GetVideoInfo().num_frames/25)*25;
}

PVideoFrame __stdcall ShowBreaks::GetFrame(int n, IScriptEnvironment *env) 
{
	if (!vi.IsYV12()) 
		env->ThrowError("ShowBreaks requires YV12.");

	PVideoFrame frame = child->GetFrame(n, env);
	if(n >= analyse->frames()/25*25)
		return frame;
	env->MakeWritable(&frame);

	const int width = frame->GetRowSize();
	const int height = frame->GetHeight();
	const int pitch = frame->GetPitch();

	double maxduplicatematch = 2.9;
	double maxblendmatch = 0.0;
	//double minweight = 0.05;


	int block = n/25;
	FindBreaks findbreaks(analyse, maxblendmatch, maxduplicatematch);
	list<double> breaks = findbreaks.interval(block*25, (block+1)*25, env);
	//for each break in [n, n+1)...
	//draw vertical line at width*(fractional part of the break)
	for(list<double>::iterator j = breaks.begin(); j != breaks.end(); ++j)
	{
		double d = *j;
		if(int(d) != n)
			continue;
		double fraction = d - int(d);
		int x = int(fraction*width);
		unsigned char *row = frame->GetWritePtr();
		for(int y = 0; y < height; ++y)
		{
			row[x] = 255;
			row += pitch;
		}
	}
	return frame;
}

Show12Breaks::Show12Breaks(LocalAnalyse *_analyse, PClip _show, IScriptEnvironment *env)
: GenericVideoFilter(_show), analyse(_analyse)
{
	//we need to be working with complete blocks of 25...
	//vi.num_frames = (_toanalyse->GetVideoInfo().num_frames/25)*25;
}

PVideoFrame __stdcall Show12Breaks::GetFrame(int n, IScriptEnvironment *env) 
{
	if (!vi.IsYV12()) 
		env->ThrowError("Show12Breaks requires YV12.");

	PVideoFrame frame = child->GetFrame(n, env);
	if(n >= analyse->frames()/25*25)
		return frame;
	env->MakeWritable(&frame);

	const int width = frame->GetRowSize();
	const int height = frame->GetHeight();
	const int pitch = frame->GetPitch();

	int block = n/25;
	FindNumberBreaks findnumberbreaks(analyse, 2.9, 12);
	list<double> breaks = findnumberbreaks.interval(block*25, (block+1)*25, env);
	//for each break in [n, n+1)...
	//draw vertical line at width*(fractional part of the break)
	for(list<double>::iterator j = breaks.begin(); j != breaks.end(); ++j)
	{
		double d = *j;
		if(int(d) != n)
			continue;
		double fraction = d - int(d);
		int x = int(fraction*width);
		unsigned char *row = frame->GetWritePtr();
		for(int y = 0; y < height; ++y)
		{
			row[x] = 255;
			row += pitch;
		}
	}
	return frame;
}

void blendlist(PClip clip, ostream &output, IScriptEnvironment *env)
{
	VideoInfo vi = clip->GetVideoInfo();
	if (!vi.IsYV12()) 
		env->ThrowError("BlendList requires YV12.");

//	double maxduplicatematch = 2.9;
	double maxblendmatch = 0.0;
	double minweight = 0.05;

	ComputeDistance distancecache(clip);
	LocalAnalyse analyse(&distancecache);

	output << "Type float\nDefault 0.0\n\n";
	for(int i = 0; i < vi.num_frames; ++i)
		if(analyse.blendmatch(i, env) < maxblendmatch)
		{
			double weight = analyse.blendweight(i, env);
			if(weight > minweight && weight < 1 - minweight)
				output << i << "\t" << weight << "\n";
		}
	
}

void blendtable(PClip clip, IScriptEnvironment *env)
{
	VideoInfo vi = clip->GetVideoInfo();
	if (!vi.IsYV12()) 
		env->ThrowError("BlendTable requires YV12.");

	ofstream table("blend table.txt");
	ofstream log("blend log.txt");

	ComputeDistance distancecache(clip);
	LocalAnalyse analyse(&distancecache);

	double maxduplicatematch = 2.9;
	double maxblendmatch = 0.0;
	double minweight = 0.05;
	for(int i = 0; i < vi.num_frames; ++i)
	{
		//write the log
		log << i <<"\t";
		if (i > 0)
			log<< distancecache.distance(i, i-1, env);
		log << "\t";
		if (i > 1)
			log<< distancecache.distance(i, i-2, env);
		log << "\n";

		//write the table
		if (i > 0 && i < vi.num_frames - 1)
		{
			if(analyse.duplicatematch(i, env) < maxduplicatematch)
				table << "<";
			if(analyse.duplicatematch(i+1, env) < maxduplicatematch)
				table << ">";
			if(analyse.blendmatch(i, env) < maxblendmatch)
			{
				double weight = analyse.blendweight(i, env);
				if(weight > minweight && weight < 1 - minweight)
					table << weight;
			}
		}
		table << "\t";
		if(i%25==24)
			table << "\n";
	}
}
//
//void blendlengths(PClip clip, IScriptEnvironment *env)
//{
//	VideoInfo vi = clip->GetVideoInfo();
//	if (!vi.IsYV12()) 
//		env->ThrowError("BlendLengths requires YV12.");
//
//	ofstream lengths("blend lengths.txt");
//}

void writebreaks(PClip clip, IScriptEnvironment *env)
//AVSValue __cdecl writebreaks(AVSValue args, void* user_data, IScriptEnvironment *env) 
{
	//PClip clip = args[0].AsClip();
	VideoInfo vi = clip->GetVideoInfo();
	if (!vi.IsYV12()) 
		env->ThrowError("WriteBreaks function requires YV12.");

	ofstream output("breaks.txt");

	ComputeDistance distancecache(clip);
	LocalAnalyse analyse(&distancecache);
	int noblocks = vi.num_frames/25;
	for(int block = 0; block < noblocks; ++block)
	{
		output << block*25 << " block:\n";
		FindNumberBreaks findnumberbreaks(&analyse, 2.9, 12);
		list<double> breaks = findnumberbreaks.interval(block*25, (block+1)*25, env);
		output << int (breaks.size()) << " breaks chosen:\n";
		for(list<double>::iterator j = breaks.begin(); j != breaks.end(); ++j)
			output << "\t"<< *j << "\n";
	}
}
