#ifndef _DISTANCE_H_
#define _DISTANCE_H_

#include <vector>
#include <map>
#include <fstream>
using namespace std;
typedef pair<int, int> intpair;

double distance2norm(const PVideoFrame &from, const PVideoFrame &to);
pair<double, double> doubledistance_C(const PVideoFrame &from, const PVideoFrame &to1, const PVideoFrame &to2);
pair<double, double> doubledistance_SSE2(const PVideoFrame &from, const PVideoFrame &to1, const PVideoFrame &to2);

class Distance//store all the distances n<->n+1, n<->n+2
{
public:
	virtual double distance(int, int, IScriptEnvironment *) /*const*/ = 0;
	virtual int frames() const = 0;
};

ostream &write(ostream &o, Distance &, IScriptEnvironment *);

struct Comparison
{
	bool operator()(intpair a, intpair b)const
		{return  a.first<b.first ||a.first == b.first && a.second <b.second;}
};

class ComputeDistance : public Distance
{
	PClip clip;
	///*mutable*/ map<intpair, double/*, Comparison*/> cache;
	struct CacheDouble 
	{
		bool found; double d;
		CacheDouble() : found(false) {}
		void set(double _d) {found = true; d = _d;}
	};
	/*mutable*/ vector<CacheDouble> cache_d1;
	/*mutable*/ vector<CacheDouble> cache_d2;

	void computepair(int n, IScriptEnvironment *env); /*const*/ //computes d(n,n+1)
	void computetriad(int n, IScriptEnvironment *env); /*const*/ //computes d(n,n+1), d(n,n+2)
public:
	ComputeDistance(/*const*/ PClip);
	virtual double distance(int, int, IScriptEnvironment *) /*const*/;
	virtual int frames() const;
};

class ReadDistance : public Distance
{
	int _frames;
//	map<intpair, double> distances;
	vector<double> d1;
	vector<double> d2;
public:
	ReadDistance(istream &);
	virtual double distance(int, int, IScriptEnvironment *) /*const*/;
	virtual int frames() const;
};
//
//class Counter
//{
//	map< string,int> count;
//public:
//	Counter(): count (0){}
//	void log (string);
//};

#endif