#include "stdafx.h"

#include "distance.h"
#include <cmath>
//#include <iomanip>
using namespace std;
//extern map<string, int> calls;



//ofstream timing("timing.txt");
//13 cycles/pixel...	double
//11 cycles/pixel...	int64
//[11 cycles/pixel...	int]
//6.6 cycles/pixel...	double accumulator
//[7 cycles/pixel...	quadruple accumulator]
//[7.7 cycles/pixel...	single accumulator, optimisation on]
//[6.8 cycles/pixel...	double accumulator, optimisation on]
//8 cycles/pixel...		calculate two 'joined' distances simultaneously
//6.4 cycles/pixel...	two distances, double accumulator
//3.3 cycles/pixel...	two distances, SSE2

double distance2norm(const PVideoFrame &from, const PVideoFrame &to)
{MeasureCycles cycles;
	const unsigned char *fromrow = from->GetReadPtr();
	const unsigned char *torow  = to->GetReadPtr();

	const int frompitch  = from->GetPitch();
	const int topitch  = to->GetPitch();
	const int width  = from->GetRowSize();
	const int height = from->GetHeight();

	__int64 sum = 0;
	for(int y = 0; y < height; ++y)
	{
		int rowsuma = 0;
		int rowsumb = 0;

		for(int x = 0; x < width; x+=2)
		{
			int differencea = fromrow[x] - torow[x];
			int differenceb = fromrow[x+1] - torow[x+1];
			rowsuma += differencea*differencea;
			rowsumb += differenceb*differenceb;
		}
		fromrow += frompitch;
		torow += topitch;

		sum += rowsuma;
		sum += rowsumb;
	}
//timing <<cycles/double(width*height) << "\n";
	return sqrt(  static_cast<double>(sum)/(width*height)  );
}



pair<double, double> doubledistance_C(const PVideoFrame &from, const PVideoFrame &to1, const PVideoFrame &to2)
{MeasureCycles cycles;
	const unsigned char *fromrow = from->GetReadPtr();
	const unsigned char *to1row  = to1->GetReadPtr();
	const unsigned char *to2row  = to2->GetReadPtr();

	const int frompitch  = from->GetPitch();
	const int to1pitch  = to1->GetPitch();
	const int to2pitch  = to2->GetPitch();
	const int width  = from->GetRowSize();
	const int height = from->GetHeight();

	__int64 sum1 = 0;
	__int64 sum2 = 0;

	for(int y = 0; y < height; ++y)
	{	
		int row1suma = 0;
		int row1sumb = 0;
		int row2suma = 0;
		int row2sumb = 0;

		for(int x = 0; x < width; x+=2)
		{
			int difference1a = fromrow[x] - to1row[x];
			int difference2a = fromrow[x] - to2row[x];
			int difference1b = fromrow[x+1] - to1row[x+1];
			int difference2b = fromrow[x+1] - to2row[x+1];
			row1suma += difference1a*difference1a;
			row2suma += difference2a*difference2a;
			row1sumb += difference1b*difference1b;
			row2sumb += difference2b*difference2b;
		}
		fromrow += frompitch;
		to1row += to1pitch;
		to2row += to2pitch;

		sum1 += row1suma;
		sum2 += row2suma;
		sum1 += row1sumb;
		sum2 += row2sumb;
	}

//timing <<cycles/double(width*height*2) << "\n";
	return make_pair(   sqrt( double(sum1)/(width*height) ), sqrt( double(sum2)/(width*height) )   );
}


pair<double, double> doubledistance_SSE2(const PVideoFrame &from, const PVideoFrame &to1, const PVideoFrame &to2)
{MeasureCycles cycles;
	const unsigned char *fromrow = from->GetReadPtr();
	const unsigned char *to1row  = to1->GetReadPtr();
	const unsigned char *to2row  = to2->GetReadPtr();

	const int frompitch  = from->GetPitch();
	const int to1pitch  = to1->GetPitch();
	const int to2pitch  = to2->GetPitch();
	const int width  = from->GetRowSize();
	const int height = from->GetHeight();

	__int64 sum1 = 0;
	__int64 sum2 = 0;

	const int widthsixteen = width - width%16;
const int blocksize = 16;
	 __asm pxor    xmm3, xmm3
	 __asm pxor    xmm7, xmm7
_declspec(align(16)) int buffer[4];

	for(int y = 0; y < height; ++y)
	{	
		__asm
		{
			push		ebx
			mov		eax, [widthsixteen]
			mov		ebx, [fromrow]
			mov		ecx, [to1row]
			mov		edx, [to2row]
			pxor		xmm4, xmm4
pixelloop:
			movq		xmm0, mmword ptr [ebx+eax-16]
			movq		xmm1, mmword ptr [ecx+eax-16]
			movq		xmm2, mmword ptr [edx+eax-16]
			punpcklbw xmm0, xmm4
			punpcklbw xmm1, xmm4
			punpcklbw xmm2, xmm4
				movq		xmm4, mmword ptr [ebx+eax-8]
				movq		xmm5, mmword ptr [ecx+eax-8]
				movq		xmm6, mmword ptr [edx+eax-8]
			psubw	xmm1, xmm0
			psubw	xmm2, xmm0
				pxor		xmm0, xmm0
				punpcklbw xmm4, xmm0
				punpcklbw xmm5, xmm0
				punpcklbw xmm6, xmm0
			pmaddwd xmm1, xmm1
			pmaddwd xmm2, xmm2
				psubw	xmm5, xmm4
				psubw	xmm6, xmm4
			paddd	xmm3, xmm1
			paddd	xmm7, xmm2
				pmaddwd xmm5, xmm5
				pmaddwd xmm6, xmm6
			pxor		xmm4, xmm4
			sub		eax, 16
				paddd	xmm3, xmm5
				paddd	xmm7, xmm6

			jnz			pixelloop
			pop		ebx
		}

		//empty xmm3 and  xmm7 often enough to prevent overflow
		if(y % blocksize == 0 || y == height-1) 
		{
			__asm movdqa    buffer, xmm3;
			for(int i = 0; i < 4; ++i)
				sum1 += buffer[i];
			__asm movdqa    buffer, xmm7;
			for(int i = 0; i < 4; ++i)
				sum2 += buffer[i];
	 __asm pxor    xmm3, xmm3
	 __asm pxor    xmm7, xmm7

		}//*/

		int row1suma = 0;
		int row1sumb = 0;
		int row2suma = 0;
		int row2sumb = 0;

		for(int x = widthsixteen; x < width; x+=2)
		{
			int difference1a = fromrow[x] - to1row[x];
			int difference2a = fromrow[x] - to2row[x];
			int difference1b = fromrow[x+1] - to1row[x+1];
			int difference2b = fromrow[x+1] - to2row[x+1];
			row1suma += difference1a*difference1a;
			row2suma += difference2a*difference2a;
			row1sumb += difference1b*difference1b;
			row2sumb += difference2b*difference2b;
		}
		fromrow += frompitch;
		to1row += to1pitch;
		to2row += to2pitch;

		sum1 += row1suma;
		sum2 += row2suma;
		sum1 += row1sumb;
		sum2 += row2sumb;
	}

//timing <<cycles/double(width*height*2) << "\n";
	return make_pair(   sqrt( double(sum1)/(width*height) ), sqrt( double(sum2)/(width*height) )   );
}


