#include "stdafx.h"

#include "findbreaks.h"
#include <vector>
#include <list>
#include <algorithm>
using namespace std;
//extern map<string, int> calls;

FindBreaks::FindBreaks(/*const*/ LocalAnalyse *_analyse, double _blendthreshold, double _duplicatethreshold)
: analyse(_analyse), blendthreshold(_blendthreshold), duplicatethreshold(_duplicatethreshold)
{
}

bool FindBreaks::isblend(int k, IScriptEnvironment *env) /*const*/
{
	double separation =0.003;
	if(analyse->blendweight(k, env) < separation || analyse->blendweight(k, env) > 1-separation)
		return false;
	//if( isduplicate(k, env) || k+1 < analyse->frames() && isduplicate(k+1, env) )//duplicates OVERRIDE blends
	//	return false;
	return analyse->blendmatch(k, env) <= blendthreshold;
}

bool FindBreaks::isduplicate(int k, IScriptEnvironment *env) /*const*/
{
	if(k == 0)//first frame cannot be a duplicate
		return false;
	
	if( isblend(k-1, env) || isblend(k, env) )//blends OVERRIDE duplicates
		return false;
	return analyse->duplicatematch(k, env) <= duplicatethreshold;
}


list<double> FindBreaks::unitinterval(int k, IScriptEnvironment *env) /*const*/
{++calls["FindBreaks::unitinterval"];
	list<double> result;
	if( isblend(k, env) )
		result.push_back(k + 1 - analyse->blendweight(k, env));
	if(	!isblend(k, env) && !isblend(k+1, env) && !isduplicate(k+1, env) )
		result.push_back(k + 1.0);
		return result;
}

list<double> FindBreaks::interval(int start, int end, IScriptEnvironment *env) /*const*/
{
++calls["FindBreaks::interval"];
	list<double> breaks;
	for(int i = start; i < end; ++i)
		breaks.splice(breaks.end(), unitinterval(i, env));
	//strip out the breaks which are less than a field away from the last break
	double lastbreak = 0.0;//it makes sense to consider this a break, though we might want another close to it...
	list<double> result;
	for(list<double>::iterator trim = breaks.begin(); trim != breaks.end(); ++trim)
		if(*trim >= lastbreak + 0.3)//MAGIC NUMBER
		{
			result.push_back(*trim);
			lastbreak = *trim;
		}

	return result;
}
	


FindNumberBreaks::FindNumberBreaks(/*const*/ LocalAnalyse *_analyse, double _duplicatethreshold, int _number)
: analyse(_analyse), duplicatethreshold(_duplicatethreshold), number(_number)
{
}
//ofstream flog("log.txt");
list<double> FindNumberBreaks::interval(int start, int end, IScriptEnvironment *env) /*const*/
{
	++calls["FindNumberBreaks::interval"];

	map<intpair, list<double> >::const_iterator i = cache.find(make_pair(start, end));
	if(i != cache.end())
		return i->second;
//timed.reset();

//	flog  <<'.';
	vector<double> thresholds;
	thresholds.reserve(end - start + 2);//0.0 and one for each of [start, end]
	thresholds.push_back(0.0);
//timed<< "in fine number breaks start";
	for(int i = start; i < end; ++i)
		thresholds.push_back(analyse->blendmatch(i, env));
	if(end < analyse->frames())
		thresholds.push_back(analyse->blendmatch(end, env));
//timed<< "in fine number breaks made";

	sort(thresholds.begin(), thresholds.end());

	//take the break sets of minimal size greater than or equal to number; pick the one with the HIGHEST threshold.
	bool firstpass = true;
	double bestthreshold = 0.0;
	int bestsize = 0;

	for(vector<double>::iterator i = thresholds.begin(); i != thresholds.end(); ++i)
	{
		FindBreaks findbreaks(analyse, *i, duplicatethreshold);
		int size = static_cast<int> (  findbreaks.interval(start, end, env).size()  );
		if(size >= number && (firstpass || size <= bestsize))
		{
			firstpass = false;
			bestthreshold = *i;
			bestsize = size;
		}
	}
	list<double> result = FindBreaks(analyse, bestthreshold, duplicatethreshold).interval(start, end, env);

	//return the first number entries of the breaks set
	list<double>::iterator discard = result.begin();
	for(int i = 0; i < number; ++i)
		++discard;
	result.erase(discard, result.end());

	cache[make_pair(start, end)] = result;
//timed<< "in fine number breaks end";

	return result;
}