#ifndef _FIND_BREAKS_H_
#define _FIND_BREAKS_H_

#include "localanalyse.h"
#include <list>
using namespace std;
/*
Initially built a class that took functions like this
	list<double> interval(int start, int end, double blendthreshold, double dupthreshold, IScriptEnvironment *env) 
but the number of arguments suggest that this may not be a natural class.
The main usage of this is to vary blendthreshold to obtain a list of a fixed size (in FindNumberBreaks)... 
convenience would suggest a class with start and end fixed.
But this seems like a very unnatural creation... to sit on a given interval and find blends in different ways.
Intuition suggests fixing the thresholds, even though this makes FindNumberBreaks more complex.
And proof comes afterwards...
FindBreaks and FindNumberBreaks become members of a general family of break finding classes
both with a 
	list<double> interval(int start, int end, IScriptEnvironment *env)
*/
class FindBreaks
{
	/*const*/ LocalAnalyse *analyse;
	double blendthreshold;
	double duplicatethreshold; 

	bool isblend(int k, IScriptEnvironment *env) /*const*/;
	bool isduplicate(int k, IScriptEnvironment *env) /*const*/;
	list<double> unitinterval(int k, IScriptEnvironment *env) /*const*/;
public:
	FindBreaks(/*const*/ LocalAnalyse *, double _blendthreshold, double _duplicatethreshold);
	list<double> interval(int start, int end, IScriptEnvironment *env) /*const*/;//find breaks in (start, end] 
		//Note that a break at 0 would be meaningless... hence the (start, end] convention.
};

class FindNumberBreaks		//find a fixed number of breaks by adjusting sensitivity
	//required to cache results for speed
{
	/*const*/ LocalAnalyse *analyse;
	//const FindBreaks findbreaks;
	double duplicatethreshold;
	int number;
	/*mutable*/ map<intpair, list<double> > cache;
public:
	FindNumberBreaks(/*const*/ LocalAnalyse *, double _duplicatethreshold, int _number);
	list<double> interval(int start, int end, IScriptEnvironment *env) /*const*/;//find breaks in (start, end]
};

#endif