#ifndef _BLEND_H_
#define _BLEND_H_

#include <fstream>
#include "types.h"

using namespace std;

class IScriptEnvironment;
class Blend//with small compact support
{
	double _FPSratio;
public:
	Blend(double __FPSratio) :_FPSratio(__FPSratio) {}
	virtual ~Blend() {}
	//virtual DisplacedVector<double> weight(int outframe) const = 0;
	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) /*const*/ = 0;
	//virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *env) const;

	double FPSratio() const {return _FPSratio;}
};

class Composition : public Blend
{
	/*const*/ Blend &b1, &b2;
public:
	Composition(/*const*/ Blend &_b1, /*const*/ Blend &_b2)//blend 1 THEN blend 2
		: Blend (_b1.FPSratio() * _b2.FPSratio()), b1(_b1), b2(_b2) {}
	
	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) /*const*/;
};

class Resample : public Blend
{
	double phase;
	double fraction;
public:
	Resample(double __inFPS, double __outFPS, double _phase)
		: Blend (__outFPS/__inFPS), phase(_phase), fraction(__inFPS/__outFPS) {}
	
	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};

class ThresholdUpsample : public Blend
{
	double phase;
	double fraction;
	double threshold;
public:
	ThresholdUpsample(double __inFPS, double __outFPS, double _phase, double _threshold)
		: Blend (__outFPS/__inFPS), phase(_phase), fraction(__inFPS/__outFPS), threshold(_threshold) {}
	
	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};

class Telecine : public Blend
{
	int skip;
public:
	Telecine(int _skip) : Blend (1.25), skip (_skip) {}

	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};

class Duplicate : public Blend
{
public:
	Duplicate() : Blend (1.0/*2.0?*/) {}

	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};

class Reblend : public Blend
{
	double lambda;
public:
	Reblend(double _lambda) : Blend (1.0), lambda(_lambda) {}

	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};

class	TestBlend : public Blend
{
public:
	TestBlend() : Blend (1.0) {}

	virtual DisplacedVector<double> weight(int outframe, IScriptEnvironment *) const;
};//*/

#endif
