#include "stdafx.h"

//#include <fstream>
#include "apply.h"
#include "Weightedaverage.h"
#include "external.h"

using namespace std;

//ofstream log("log.txt");

void setFPS(VideoInfo &vi, double FPS)
{
	double n = FPS;
	int d = 1;
	while (n < 16777216 && d < 16777216) 
		n*=2, d*=2;
	vi.SetFPS( int(n+0.5),d);
}

ForwardBlend::ForwardBlend(PClip _child, /*const*/ Blend &_blend, IScriptEnvironment* env)
: GenericVideoFilter(_child), blend(_blend)
{
	int scan = 0;
	while (blend.weight(scan, env).endindex()-1 < vi.num_frames)
		++scan;
	vi.num_frames = scan;

	setFPS(vi, double(vi.fps_numerator) / vi.fps_denominator * blend.FPSratio());
}

PVideoFrame __stdcall ForwardBlend::GetFrame(int n, IScriptEnvironment* env)
{
	return WeightedAverage(child, blend.weight(n, env), env, vi);
}


ReverseBlend::ReverseBlend(PClip _child, /*const */Blend &_blend, IScriptEnvironment* env)
: GenericVideoFilter(_child), blend(_blend)
{
	blendedframecount = vi.num_frames;
	originalframecount = blend.weight(blendedframecount-1, env).endindex();

	vi.num_frames = originalframecount;
	setFPS(vi, double(vi.fps_numerator) / vi.fps_denominator / blend.FPSratio());
}
		//ofstream log("logspoke to.txt");

PVideoFrame __stdcall ReverseBlend::GetFrame(int n, IScriptEnvironment* env) 
{
	if (!vi.IsYV12()) 
		env->ThrowError("ReverseBlend requires YV12.");
	if(n >= vi.num_frames)
	{
		env->ThrowError("ReverseBlend: invalid frame number requested.");
	}

	try
	{
		//first find a blended frame which depends on frame n
		int guessblended = int(n * blend.FPSratio() + 0.5);
		if(guessblended >= blendedframecount)
			guessblended = blendedframecount - 1;
		if(blend.weight(guessblended, env).endindex()-1 < n)
		{
			while(guessblended < blendedframecount && blend.weight(guessblended, env).endindex()-1 < n)
				++guessblended;
			if(guessblended == blendedframecount)
				return child->GetFrame(0, env);//no blended frames contain data from the requested frame
		}
		else if(blend.weight(guessblended, env).beginindex() > n)
		{
			while(guessblended > 0 && blend.weight(guessblended, env).beginindex() > n)
				--guessblended;
			if(blend.weight(guessblended, env).beginindex() > n)
				return child->GetFrame(0, env);//no blended frames contain data from the requested frame
		}

		//find surrounding frames that can be computed trivially
		int firstblended = guessblended;		
		while(firstblended+1 < blendedframecount && blend.weight(firstblended+1, env).definedat(n))
			++firstblended;	//to make sure all the frames depending on n are considered...
		while(firstblended > 0 && blend.weight(firstblended, env).size() != 1)
			--firstblended;

		int lastblended = guessblended;
		while(lastblended-1 >= 0 && blend.weight(lastblended-1, env).definedat(n))
			--lastblended;	//to make sure all the frames depending on n are considered...
		while(lastblended < blendedframecount-1 && blend.weight(lastblended, env).size() != 1)
			++lastblended;

		//note that we can have the case firstblended > lastblended
		//in the case where more than one blended frame is a copy of original frame n
		//in this case frames [lastblended, firstblended] are all copies of n.
		if(lastblended <= firstblended)
			return child->GetFrame(lastblended, env);

		int firstoriginal = blend.weight(firstblended, env).beginindex();
		int lastoriginal = blend.weight(lastblended, env).endindex()-1;
			
		int noblended = lastblended - firstblended + 1;
		int nooriginal = lastoriginal - firstoriginal + 1;

		ColumnMatrix<double> A(noblended, nooriginal, 0.0);
		for(int i = firstblended; i <= lastblended; ++i)
		{
			DisplacedVector<double> weight=blend.weight(i, env);
			for (int j = weight.beginindex(); j < weight.endindex(); ++j)
				A(i - firstblended, j - firstoriginal) = weight.at(j);
		}

		ColumnMatrix<double> p = pseudoinverse(A);

		DisplacedVector<double> reverseweight(firstblended, lastblended + 1);
		for(int i = firstblended; i <= lastblended; ++i)
			reverseweight.at(i) = p (n - firstoriginal, i - firstblended);
		return WeightedAverage(child, reverseweight, env, vi);
	}
	catch(...)
	{
		env->ThrowError("ReverseBlend: error in ReverseBlend::GetFrame.");
		throw;//to preventcompiler warning
	}
}

